﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;

namespace XamDataChartServiceReportsCase
{
    /// <summary>
    /// Interaction logic for SeriesBinder.xaml
    /// </summary>
    public partial class SeriesBinder : Window
    {
        public SeriesBinder()
        {
            InitializeComponent();
            DataContext = new MainViewModel(); 
        }

        #region Series Binder

        public  class SeriesBinderBehavior : DependencyObject
        {
            public static readonly DependencyProperty SeriesBinderProperty =
                DependencyProperty.RegisterAttached(
                    "SeriesBinder",
                    typeof(SeriesBinderInfo),
                    typeof(SeriesBinderBehavior),
                    new PropertyMetadata(null,
                        (o, e) =>
                        {
                            OnSeriesBiderChaged(o as XamDataChart,
                                e.OldValue as SeriesBinderInfo,
                                e.NewValue as SeriesBinderInfo);
                        }));

            public static SeriesBinderInfo GetSeriesBinder(DependencyObject target)
            {
                return target.GetValue(SeriesBinderProperty) as SeriesBinderInfo;
            }

            public static void SetSeriesBinder(DependencyObject target, object value)
            {
                target.SetValue(SeriesBinderProperty, value);
            }


            private static void OnSeriesBiderChaged(
                XamDataChart chart,
                SeriesBinderInfo oldVal,
                SeriesBinderInfo newVal)
            {
                if (chart == null)
                {
                    return;
                }
                if (oldVal != null)
                {
                    oldVal.DetachOwner(chart);
                }
                if (newVal != null)
                {
                    newVal.AttachOwner(chart);
                }
            }
        }

    

        #endregion

        #region View Model
        public class MainViewModel : PropertyChangedNotifier
        {
            private ObservableCollection<SeriesViewModel> seriesSource;

            public MainViewModel()
            {
                SeriesSource = new ObservableCollection<SeriesViewModel>
            {  
                new SeriesViewModel
                {
                    Source = ChartData.GenerateData(10),
                    Type = SeriesType.ColumnSeries,
                    XPath = "Date",
                    YPath = "Value",
                },
                new SeriesViewModel
                {
                    Source = ChartData.GenerateData(22),
                    Type = SeriesType.LineSeries,
                    XPath = "Label",
                    YPath = "Value",
                },
              
                new SeriesViewModel
                {
                    Source = ChartData.GenerateData(20),
                    Type = SeriesType.ScatterSeries,
                    XPath = "Index",
                    YPath = "Value",
                },
                new SeriesViewModel
                {
                    Source = ChartData.GenerateData(15),
                    Type = SeriesType.ScatterLineSeries,
                    XPath = "Index",
                    YPath = "Value",
                }, 
            };
            }

            public ObservableCollection<SeriesViewModel> SeriesSource
            {
                get { return seriesSource; }
                set { seriesSource = value; OnPropertyChagned("SeriesSource"); }
            }

            public ICommand AddSeries
            {
                get
                {
                    return new RelayCommand<object>(
                        new Action<object>(
                            (obj) =>
                            {
                                SeriesSource.Add(
                                    new SeriesViewModel
                                    {
                                        Source = ChartData.GenerateData(10),
                                        Type = SeriesType.ScatterLineSeries,
                                        XPath = "Index",
                                        YPath = "Value",
                                    });
                            }));
                }
            }

            public ICommand RemoveSeries
            {
                get
                {
                    return new RelayCommand<object>(
                        new Action<object>(
                            (obj) =>
                            {
                                if (SeriesSource.Count > 0)
                                    SeriesSource.RemoveAt(0);
                            }));
                }
            }

            public ICommand ClearSeries
            {
                get
                {
                    return new RelayCommand<object>(
                        new Action<object>(
                            (obj) =>
                            {
                                SeriesSource.Clear();
                            }));
                }
            }

            public ICommand ResetSeries
            {
                get
                {
                    return new RelayCommand<object>(
                        new Action<object>(
                            (obj) =>
                            {
                                SeriesSource = new ObservableCollection<SeriesViewModel>
                            {  
                                new SeriesViewModel
                                {
                                    Source = ChartData.GenerateData(10),
                                    Type = SeriesType.ColumnSeries,
                                    XPath = "Date",
                                    YPath = "Value",
                                },
                                new SeriesViewModel
                                {
                                    Source = ChartData.GenerateData(22),
                                    Type = SeriesType.LineSeries,
                                    XPath = "Label",
                                    YPath = "Value",
                                },
              
                                new SeriesViewModel
                                {
                                    Source = ChartData.GenerateData(20),
                                    Type = SeriesType.ScatterSeries,
                                    XPath = "Index",
                                    YPath = "Value",
                                },
                                new SeriesViewModel
                                {
                                    Source = ChartData.GenerateData(15),
                                    Type = SeriesType.ScatterLineSeries,
                                    XPath = "Index",
                                    YPath = "Value",
                                }, 
                            };
                            }));
                }
            }
        }

        public class SeriesViewModel : PropertyChangedNotifier
        {
            private SeriesType type;
            public SeriesType Type
            {
                get { return type; }
                set { type = value; OnPropertyChagned("Type"); }
            }

            private IEnumerable source;
            public IEnumerable Source
            {
                get { return source; }
                set { source = value; OnPropertyChagned("Source"); }
            }

            private string xPath;
            public string XPath
            {
                get { return xPath; }
                set { xPath = value; OnPropertyChagned("XPath"); }
            }

            private string yPath;
            public string YPath
            {
                get { return yPath; }
                set { yPath = value; OnPropertyChagned("XPath"); }
            }
        }

        public class ChartData
        {
            public DateTime Date { get; set; }
            public int Index { get; set; }
            public string Label { get; set; }
            public double Value { get; set; }

            public static ObservableCollection<ChartData> GenerateData(int count)
            {
                ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

                Random r = new Random();

                ChartData data0 = new ChartData
                {
                    Date = DateTime.Now,
                    Index = 0,
                    Value = r.Next(20, 40),
                    Label = "Item 0"
                };

                data.Add(data0);

                for (int i = 1; i < count; i++)
                {
                    ChartData dat = new ChartData
                    {
                        Date = DateTime.Now.AddDays(i),
                        Index = i,
                        Label = "Item " + i,
                    };

                    if (r.NextDouble() > .5)
                        dat.Value = data[i - 1].Value + r.NextDouble();
                    else
                        dat.Value = data[i - 1].Value - r.NextDouble();

                    data.Add(dat);
                }

                return data;
            }
        }

        public class PropertyChangedNotifier : INotifyPropertyChanged
        {
            public event PropertyChangedEventHandler PropertyChanged;

            public void OnPropertyChagned(string name)
            {
                if (PropertyChanged != null)
                    PropertyChanged(this, new PropertyChangedEventArgs(name));
            }
        }

        public enum SeriesType
        {
            LineSeries,
            ColumnSeries,
            ScatterSeries,
            ScatterLineSeries,
        }

        public class RelayCommand<T> : ICommand
        {

            #region Declarations

            readonly Predicate<T> _canExecute;
            readonly Action<T> _execute;

            #endregion

            #region Constructors

            /// <summary>
            /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class and the command can always be executed.
            /// </summary>
            /// <param name="execute">The execution logic.</param>
            public RelayCommand(Action<T> execute)
                : this(execute, null)
            {
            }

            /// <summary>
            /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class.
            /// </summary>
            /// <param name="execute">The execution logic.</param>
            /// <param name="canExecute">The execution status logic.</param>
            public RelayCommand(Action<T> execute, Predicate<T> canExecute)
            {

                if (execute == null)
                    throw new ArgumentNullException("execute");
                _execute = execute;
                _canExecute = canExecute;
            }

            #endregion

            #region ICommand Members

            public event EventHandler CanExecuteChanged
            {
                add
                {

                    if (_canExecute != null)
                        CommandManager.RequerySuggested += value;
                }
                remove
                {

                    if (_canExecute != null)
                        CommandManager.RequerySuggested -= value;
                }
            }

            [DebuggerStepThrough]
            public virtual Boolean CanExecute(Object parameter)
            {
                return _canExecute == null ? true : _canExecute((T)parameter);
            }

            public virtual void Execute(Object parameter)
            {
                _execute((T)parameter);
            }

            #endregion
        }


        public class Test
        {
            public string empty { get; set; }
        }

        #endregion
    }
}
